# workflow.py
# Workflow automation with PLAMS

import os
import sys
sys.path.append(os.getcwd()) # make sure we can import from the current dir
import my_settings # my_settings.py contains some predefined settings

def get_good_excitations(job, energy, osc_str):
  """Returns the excitations satisfying the condition e < en_thresh and o > osc_thresh. 
     Units: energies in eV and oscillator strengths in Debye"""
  good_excitations = []
  # Before reading the results, make sure that the job terminated normally
  if job.check():
    # The results on the binary KF file are in atomic units
    exc = [Units.convert(e,'au','eV') 
           for e in job.results.readkf('Excitations SS A','excenergies')]
    osc = [Units.convert(o,'au','Debye') 
           for o in job.results.readkf('Excitations SS A','oscillator strengths')]
    for e,o in zip(exc,osc):
      # Check whether the excitation satisfies our condition: 
      if e < energy and o > osc_str: 
        good_excitations.append((e,o))
  return good_excitations

@add_to_class(DFTBResults)
def get_opt_molecule(self):
   return self.get_molecule('Molecule','Coords')

@add_to_class(ADFResults)
def get_opt_molecule(self):
   return self.get_molecule('Geometry','xyz')

################################ Step 0 ################################

# Create a list of Molecules from the .xyz files in the directory 'molecules':

molecules_dir = 'molecules'

molecules = []
# Loop over all the file in the directory
for mol_name in os.listdir(molecules_dir):
  mol = Molecule(os.path.join(molecules_dir,mol_name))
  # the properties attribute of the Molecule object is a PLAMS’ Settings object
  mol.properties.name = os.path.splitext(mol_name)[0]
  molecules.append(mol)

################################ Step 1 ################################

print("Step 1: optimize molecules and calculate excitations with DFTB")

promising_molecules = []
for mol in molecules:
  # The settings are defined in the module ``my_settings``, 
  # which we imported at the beginning of our script
  dftb_job = DFTBJob(molecule=mol, settings=my_settings.dftb_GO_plus_excitations(), 
                     name='DFTB_'+mol.properties.name)
  dftb_job.run()
  good_excitations = get_good_excitations(dftb_job, energy=6.0, osc_str=1.0E-4)
  if good_excitations: 
    print ("Found a molecule with a promising excitation: " + mol.properties.name)
    promising_molecules.append(dftb_job.results.get_opt_molecule())

print("Number of promising molecules: %i " % len(promising_molecules))

################################ Step 2 ################################

print("Step 2: Geometry optimizations with ADF")

optimized_molecules = []
for mol in promising_molecules:
  adf_GO_job = ADFJob(molecule=mol, settings=my_settings.adf_GO(basis='DZP', XC='LDA'), 
                      name='ADF_GO_'+mol.properties.name)
  adf_GO_job.run()

  # Before reading the results, make sure that the job terminated normally
  if adf_GO_job.check(): 
    optimized_molecules.append(adf_GO_job.results.get_opt_molecule())

################################ Step 3 ################################

print("Step 3: Calculate the excitations with ADF")

for mol in optimized_molecules:
  adf_exci_job = ADFJob(molecule=mol, settings=my_settings.adf_excitations(n_excitations=5), 
                        name='ADF_EXCI_'+mol.properties.name)
  adf_exci_job.run()
  good_excitations = get_good_excitations(adf_exci_job, energy=4.0, osc_str=1.0E-4)
  if good_excitations: 
    print ("Found a molecule with a good excitation: " + mol.properties.name)
    print ("Excitations (energy [eV], oscillator strength [Debye]:" + str(good_excitations))